#if __VERSION__ == 100
precision mediump float;
#endif

//params is a 4-part system for determining how to draw the scene
// - x is whether or not to use a single color for drawing
// - y is whether or not to use the varying color for drawing
// - z is the number of texture units to use
// - w is the blend mode to use
uniform vec4 params;

//paintAttrib is a list of opacity, brightness, and saturation values
// - x is opacity
// - y is brightness
// - z is saturation
uniform vec3 paintAttrib;

uniform sampler2D texture0;
uniform sampler2D texture1;
uniform sampler2D texture2;
uniform sampler2D texture3;

uniform vec4 singleColor;

varying vec4 vColor;
varying vec2 vTexCoord0;
varying vec2 vTexCoord1;
varying vec2 vTexCoord2;
varying vec2 vTexCoord3;

void main () {
    vec4 color = vec4 (1, 1, 1, 1);

    if (params.x == 1.0)
	color *= singleColor;
    if (params.y == 1.0)
	color *= vColor;

    if (params.z == 0.25)
	color *= texture2D (texture0, vTexCoord0);

    if (paintAttrib.z != 1.0)
    {
	vec3 desaturated = color.rgb * vec3 (0.30, 0.59, 0.11);
	desaturated = vec3 (dot (desaturated, color.rgb));
	color.rgb = color.rgb * vec3 (paintAttrib.z) + desaturated *
	                                            vec3 (1.0 - paintAttrib.z);
    }

    color.rgb = color.rgb * paintAttrib.x * paintAttrib.y;
    color.a = color.a * paintAttrib.x;

    gl_FragColor = color;
}

